extern crate chrono;
extern crate colored;
extern crate dirs;
extern crate getopts;
#[macro_use]
extern crate log;
extern crate serde;
#[macro_use]
extern crate serde_derive;
extern crate toml;

mod app;
mod apps;
mod config;
mod logger;
mod proc;
mod result;

use std::path::PathBuf;

use apps::Apps;
use config::Config;
use proc::Proc;
use result::Result;

fn main() {
	let config = Config::from_args();

	if let Err(e) = logger::init() {
		println!("FATAL: Error initializing logger: {:?}", e);
		std::process::exit(1);
	}
	logger::set_level(config.log_level);

	let mut apps_file = config.config_directory.clone();
	apps_file.push("apps.toml");

	match startup(apps_file) {
		Ok(p) => {
			info!("shutting down");
			for mut proc in p {
				proc.stop();
			}
		}
		Err(e) => {
			error!("FATAL: {:?}", e);
			std::process::exit(1);
		}
	};
}

fn startup(apps_file: PathBuf) -> Result<Vec<Proc>> {
	let apps = Apps::load(apps_file)?;
	let (procs, holds) = apps.start();
	let procs = apps.run(procs, holds)?;
	Ok(procs)
}
